from django.db import models

# Create your models here.
class Musician(models.Model):
    first_name = models.CharField(max_length=50,null=True)
    last_name = models.CharField(max_length=50,null=True)
    instrument = models.CharField(max_length=100,null=True)

class Order(models.Model):
    staff = models.CharField(max_length=50,null=True)
    proyecto = models.CharField(max_length=50,null=True)
    subproyecto = models.CharField(max_length=50,null=True)
    operacion = models.CharField(max_length=50,null=True)
    sector = models.CharField(max_length=50,null=True)
    activo = models.CharField(max_length=50,null=True)
    requisitar = models.CharField(max_length=50,null=True)

    complete = models.BooleanField(null=True)

    sol_autorizada_por = models.CharField(max_length=50,null=True)
    autorizar = models.BooleanField(null=True, default=None)
    created_at = models.DateTimeField(auto_now_add=True)
    approved_at = models.DateField(auto_now=True)
    approved_at_time = models.TimeField(auto_now=True)


    def __str__(self):
        return f'{self.get_folio} ordered by {self.staff}'


    @property
    def get_cart_total(self):
        productos = self.articulosordenados_set.all()
        total = sum([producto.get_total for producto in productos])
        return total

    @property
    def get_cart_quantity(self):
        productos = self.articulosordenados_set.all()
        total = sum([producto.cantidad for producto in productos])
        return total

    @property
    def get_folio(self):
        return str(self.pk).zfill(6)

class Inventario(models.Model):
    codigo = models.CharField(max_length=50,null=True)
    nombre = models.CharField(max_length=50,null=True)
    subproyecto = models.CharField(max_length=50,null=True)
    operacion = models.CharField(max_length=50,null=True)
    sector = models.CharField(max_length=50,null=True)
    activo = models.CharField(max_length=50,null=True)
    staff = models.CharField(max_length=50,null=True)

    complete = models.BooleanField(null=True)

    sol_autorizada_por = models.CharField(max_length=50,null=True)
    autorizar = models.BooleanField(null=True, default=None)
    created_at = models.DateTimeField(auto_now_add=True)
    approved_at = models.DateField(auto_now=True)
    approved_at_time = models.TimeField(auto_now=True)


    def __str__(self):
        return f'{self.get_folio} ordered by {self.staff}'


    @property
    def get_cart_total(self):
        productos = self.articulosordenados_set.all()
        total = sum([producto.get_total for producto in productos])
        return total

    @property
    def get_cart_quantity(self):
        productos = self.articulosordenados_set.all()
        total = sum([producto.cantidad for producto in productos])
        return total

    @property
    def get_folio(self):
        return str(self.pk).zfill(6)


class ArticulosOrdenados(models.Model):
    producto = models.ForeignKey(Inventario, on_delete = models.CASCADE, null=True)
    orden = models.ForeignKey(Order, on_delete = models.CASCADE, null=True)
    cantidad = models.IntegerField(default=0, null=True, blank=True)

    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f'{self.orden} - {self.producto}'

    @property
    def get_total(self):
        total = self.producto.price * self.cantidad
        return total

class ArticulosparaSurtir(models.Model):
    articulos = models.ForeignKey(ArticulosOrdenados, on_delete = models.CASCADE, null=True)
    cantidad = models.IntegerField(default=0, null=True, blank= True)
    surtir = models.BooleanField(null=True, default=False)
    cantidad_requisitar = models.IntegerField(default=0, null=True, blank=True)
    requisitar = models.BooleanField(null=True, default=False)
    salida = models.BooleanField(null=True, default=False)
    history = models.CharField(max_length=50,null=True)
    firma = models.BooleanField(null=True, default=False)



    created_at = models.DateTimeField(auto_now_add=True)
    modified_at = models.DateTimeField(auto_now=True)


    def __str__(self):
        return f'{self.articulos} - {self.cantidad} - {self.cantidad_requisitar}'

class Independientes(models.Model):
    NUMERO = models.CharField(max_length=50,null=True)
    EMPRESA = models.CharField(max_length=50,null=True)
    DISTRITO = models.CharField(max_length=50,null=True)
    SECTOR = models.CharField(max_length=50,null=True)
    SECCION = models.CharField(max_length=50,null=True)
    NOMBRE = models.CharField(max_length=50,null=True)
    TIPO_DE_CONTRATO = models.CharField(max_length=50,null=True)
    PUESTO = models.CharField(max_length=50,null=True)
    AMORTIZACION_INFONAVIT = models.CharField(max_length=50,null=True)
    FONACOT = models.CharField(max_length=50,null=True)
    #TOTAL DEDUCCIÓN = models.CharField(max_length=50,null=True)
    #NETO A PAGAR = models.CharField(max_length=50,null=True)
    Neto_Catorcenal_SIN_DEDUCCIONES = models.CharField(max_length=50,null=True)
    #SUELDO MENSUAL NETO = models.CharField(max_length=50,null=True)
    COMPLEMENTO_DE_SALARIO_CATORCENAL = models.CharField(max_length=50,null=True)
    #COMPLEMENTO DE SALARIO MENSUAL = models.CharField(max_length=50,null=True)
    SUELDO_DIARIO = models.CharField(max_length=50,null=True)
    SDI = models.CharField(max_length=50,null=True)
    #SUELDO MENSUAL = models.CharField(max_length=50,null=True)
    #SSUELDO MENSUAL SDI = models.CharField(max_length=50,null=True)
    APOYO_DE_PASAJES = models.CharField(max_length=50,null=True)
    #TOTAL PERCEPCIONES MENSUAL = models.CharField(max_length=50,null=True)
    #IMPUESTO ESTATAL = models.CharField(max_length=50,null=True)
    IMSS_OBRERO_PATRONAL = models.CharField(max_length=50,null=True)
    #SAR 2% = models.CharField(max_length=50,null=True)
    #CESANTIA 4.625% = models.CharField(max_length=50,null=True)
    #INFONAVIT 5% = models.CharField(max_length=50,null=True)
    #ISR = models.CharField(max_length=50,null=True)
    #-----Lim. Inferior = models.CharField(max_length=50,null=True)
    #Excedente = models.CharField(max_length=50,null=True)
    #-----Tasa = models.CharField(max_length=50,null=True)
    #Impuesto marginal = models.CharField(max_length=50,null=True)
    #-----Cuota fija = models.CharField(max_length=50,null=True)
    #Impuesto = models.CharField(max_length=50,null=True)
    #-----Subsidio = models.CharField(max_length=50,null=True)
    APOYO_DE_VISITA_FAM = models.CharField(max_length=50,null=True)
    ESTANCIA = models.CharField(max_length=50,null=True)
    RENTA = models.CharField(max_length=50,null=True)
    APOYO_DE_ESTUDIOS = models.CharField(max_length=50,null=True)
    AMV = models.CharField(max_length=50,null=True)
    CAMPAMENTO = models.CharField(max_length=50,null=True)
    #OTROS BONOS = models.CharField(max_length=50,null=True)
    #TOTAL DE APOYOS Y BONOS (empleado comprueba) = models.CharField(max_length=50,null=True)
    #TOTAL DE APOYOS Y BONOS (agregar comisión) = models.CharField(max_length=50,null=True)
    #COMISION 8% COMPLEMENTO DE SALARIO Y BONOS = models.CharField(max_length=50,null=True)
    #TOTAL COSTO MENSUAL PARA LA EMPRESA = models.CharField(max_length=50,null=True)
    BONOS_CT_OCHO = models.CharField(max_length=50,null=True)
    BONOS_CT_NUEVE = models.CharField(max_length=50,null=True)
    #INGRESO MENSUAL NETO DEL EMPLEADO = models.CharField(max_length=50,null=True)
    def __str__(self):
        return f'{self.NUMERO} belonging to {self.EMPRESA}'

class LimInf(models.Model):
    #Limite Inferior
    A = models.CharField(max_length=50,null=True)
    def __str__(self):
        return f'{self.A}'
    @property
    def get_cart_total(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.get_total for objeto in objetos])
        return total

    @property
    def get_cart_quantity(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.cantidad for objeto in objetos])
        return total

    @property
    def get_folio(self):
        return str(self.pk).zfill(6)

class LimSup(models.Model):
    #Limite Inferior
    B = models.CharField(max_length=50,null=True)
    def __str__(self):
        return f'{self.B}'
    @property
    def get_cart_total(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.get_total for objeto in objetos])
        return total

    @property
    def get_cart_quantity(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.cantidad for objeto in objetos])
        return total

    @property
    def get_folio(self):
        return str(self.pk).zfill(6)

class Cuota(models.Model):
    #Limite Inferior
    C = models.CharField(max_length=50,null=True)
    def __str__(self):
        return f'{self.C}'
    @property
    def get_cart_total(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.get_total for objeto in objetos])
        return total

    @property
    def get_cart_quantity(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.cantidad for objeto in objetos])
        return total

    @property
    def get_folio(self):
        return str(self.pk).zfill(6)

class Excedente(models.Model):
    #Limite Inferior
    D = models.CharField(max_length=50,null=True)
    def __str__(self):
        return f'{self.D}'
    @property
    def get_cart_total(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.get_total for objeto in objetos])
        return total

    @property
    def get_cart_quantity(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.cantidad for objeto in objetos])
        return total

    @property
    def get_folio(self):
        return str(self.pk).zfill(6)

class PIngresos(models.Model):
    #Limite Inferior
    E = models.CharField(max_length=50,null=True)
    def __str__(self):
        return f'{self.E}'
    @property
    def get_cart_total(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.get_total for objeto in objetos])
        return total

    @property
    def get_cart_quantity(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.cantidad for objeto in objetos])
        return total

    @property
    def get_folio(self):
        return str(self.pk).zfill(6)

class HIngresos(models.Model):
    #Limite Inferior
    F = models.CharField(max_length=50,null=True)
    def __str__(self):
        return f'{self.F}'
    @property
    def get_cart_total(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.get_total for objeto in objetos])
        return total

    @property
    def get_cart_quantity(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.cantidad for objeto in objetos])
        return total

    @property
    def get_folio(self):
        return str(self.pk).zfill(6)

class Subsidio(models.Model):
    #Limite Inferior
    G = models.CharField(max_length=50,null=True)
    def __str__(self):
        return f'{self.G}'
    @property
    def get_cart_total(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.get_total for objeto in objetos])
        return total

    @property
    def get_cart_quantity(self):
        objetos = self.DatosISR.objects_set.all()
        total = sum([objeto.cantidad for objeto in objetos])
        return total

    @property
    def get_folio(self):
        return str(self.pk).zfill(6)

class DatosISR(models.Model):
    LimInf = models.ForeignKey(LimInf, on_delete = models.CASCADE, null=True)
    LimSup = models.ForeignKey(LimSup, on_delete = models.CASCADE, null=True)
    Cuota = models.ForeignKey(Cuota, on_delete = models.CASCADE, null=True)
    Excedente = models.ForeignKey(Excedente, on_delete = models.CASCADE, null=True)
    PIngresos = models.ForeignKey(PIngresos, on_delete = models.CASCADE, null=True)
    HIngresos = models.ForeignKey(HIngresos, on_delete = models.CASCADE, null=True)
    Subsidio = models.ForeignKey(Subsidio, on_delete = models.CASCADE, null=True)
    complete = models.BooleanField(null=True)
    def __str__(self):
        return f'Complete information?: {self.complete}'




